// src/index.ts
import plugin from "tailwindcss/plugin";

// src/utils/utils.ts
function simpleHash(str) {
  let hash = 0;
  for (let i = 0; i < str.length; i++) {
    const char = str.charCodeAt(i);
    hash = (hash << 5) - hash + char;
  }
  return (hash >>> 0).toString(36).padStart(7, "0");
}
function arraySum(arr) {
  return arr.reduce((sum, a) => sum + a, 0);
}
function flattenColorPalette(colors = {}) {
  return Object.assign({}, ...Object.entries(colors).flatMap(([color, values]) => typeof values == "object" ? Object.entries(flattenColorPalette(values)).map(([number, hex]) => ({
    [color + (number === "DEFAULT" ? "" : `-${number}`)]: hex
  })) : [
    {
      [`${color}`]: values
    }
  ]));
}

// src/index.ts
var defaultOptions = {
  delimiter: ";",
  typeLetterDuration: 0.1,
  pauseAfterWordDuration: 2,
  deleteLetterDuration: 0.05,
  pauseAfterDeletionDuration: 1
};
var ZERO_WIDTH_SPACE = "\u200B";
var src_default = plugin.withOptions(
  (options) => ({ addComponents, matchComponents, matchUtilities, theme }) => {
    const optionsWithDefaults = Object.assign(defaultOptions, options);
    matchComponents(
      {
        typed: (text) => {
          const hash = simpleHash(text);
          const strings = text.split(optionsWithDefaults.delimiter);
          const durations = strings.map((string) => string.length * (optionsWithDefaults.typeLetterDuration + optionsWithDefaults.deleteLetterDuration) + optionsWithDefaults.pauseAfterWordDuration + optionsWithDefaults.pauseAfterDeletionDuration);
          const durationsCummulative = durations.map((_dur, durIdx) => arraySum(durations.slice(0, durIdx + 1)));
          const duration = arraySum(durations);
          function getTypingKeyframeStep() {
            const keyframes = [];
            strings.forEach((string, stringIdx) => {
              [...string].forEach((_char, charIdx) => {
                const isLastString = stringIdx + 1 === strings.length;
                const isLastChar = charIdx + 1 === string.length;
                const durationType = (durationsCummulative[stringIdx - 1] || 0) + charIdx * optionsWithDefaults.typeLetterDuration;
                const durationDelete = (durationsCummulative[stringIdx - 1] || 0) + string.length * optionsWithDefaults.typeLetterDuration + optionsWithDefaults.pauseAfterWordDuration + (string.length - charIdx) * optionsWithDefaults.deleteLetterDuration;
                const keyframe = [`${durationType / duration * 100}%, ${durationDelete / duration * 100}%`, { content: `"${string.slice(0, charIdx + 1)}"` }];
                keyframes.push(keyframe);
                if (isLastChar) {
                  const durationType2 = durationsCummulative[stringIdx] - optionsWithDefaults.pauseAfterDeletionDuration + optionsWithDefaults.typeLetterDuration;
                  const offsetTrick = -(Number(!isLastString) * 1e-4);
                  const durationDelete2 = durationsCummulative[stringIdx] + offsetTrick;
                  const pauseKeyframe = [`${durationType2 / duration * 100}%, ${durationDelete2 / duration * 100}%`, { content: `"${ZERO_WIDTH_SPACE}"` }];
                  keyframes.push(pauseKeyframe);
                }
              });
            });
            return Object.fromEntries(keyframes);
          }
          return {
            "--tw-typed-typing-duration": `${duration}s`,
            "--tw-typed-typing-delay": "0s",
            "&::before": {
              content: `"${strings.join(optionsWithDefaults.delimiter)}"`,
              whiteSpace: "break-spaces",
              willChange: "content",
              animation: `tw-typed-typing-${hash} var(--tw-typed-typing-duration) linear var(--tw-typed-typing-delay) infinite`
            },
            [`@keyframes tw-typed-typing-${hash}`]: {
              ...getTypingKeyframeStep()
            }
          };
        }
      }
    );
    addComponents(
      {
        ".typed-caret": {
          "--tw-typed-caret-content": `"${ZERO_WIDTH_SPACE}"`,
          "--tw-typed-caret-color": "currentcolor",
          "--tw-typed-caret-width": "0.2ch",
          "--tw-typed-caret-space": "0.2ch",
          "--tw-typed-caret-duration": "1s",
          "--tw-typed-caret-delay": "0s",
          "&::after": {
            content: "var(--tw-typed-caret-content)",
            paddingRight: "var(--tw-typed-caret-space)",
            borderRight: "var(--tw-typed-caret-width) solid var(--tw-typed-caret-color)",
            animation: "tw-typed-caret var(--tw-typed-caret-duration) linear var(--tw-typed-caret-delay) infinite"
          },
          "@keyframes tw-typed-caret": {
            "0%, 20%, 80%, 100%": { opacity: "1" },
            "30%, 70%": { opacity: "0" }
          }
        }
      }
    );
    matchUtilities(
      {
        "typed-caret-color": (value) => ({
          "--tw-typed-caret-color": value
        })
      },
      {
        type: ["color"],
        values: flattenColorPalette(theme("colors"))
      }
    );
    matchUtilities(
      {
        "typed-caret-width": (value) => ({
          "--tw-typed-caret-width": value
        })
      },
      {
        values: theme("spacing")
      }
    );
    matchUtilities(
      {
        "typed-caret-space": (value) => ({
          "--tw-typed-caret-space": value
        })
      },
      {
        values: theme("spacing")
      }
    );
  }
);
export {
  src_default as default
};
